@@ -5,7 +5,7 @@
 from synalinks.src import testing
 from synalinks.src.backend import DataModel
 from synalinks.src.callbacks import Callback
-from synalinks.src.optimizers import RandomFewShot
+from synalinks.src.optimizers.random_few_shot import RandomFewShot
 from synalinks.src.programs import Program
 from synalinks.src.rewards import ExactMatch
 
