@@ -2,7 +2,6 @@
 
 import asyncio
 import uuid
-from typing import List
 
 from synalinks.src import ops
 from synalinks.src.api_export import synalinks_export
@@ -15,23 +14,18 @@ from synalinks.src.backend import ToolCall
 from synalinks.src.backend import is_chat_messages
 from synalinks.src.backend.common.dynamic_json_schema_utils import dynamic_tool_calls
 from synalinks.src.backend.common.json_utils import out_mask_json
-from synalinks.src.modules.core.generator import Generator
 from synalinks.src.modules.module import Module
 from synalinks.src.modules.ttc.chain_of_thought import ChainOfThought
 from synalinks.src.saving import serialization_lib
 
 
-def get_default_static_prompt():
-    return "You are a helpful assistant"
-
-
-def get_default_instructions() -> List[str]:
-    """The default parallel agent instructions."""
-    return [
-        "Think step by step: Use the thinking field to elaborate what you observe and what do you need to accomplish next",
-        "Reflect on prior steps: Review your previous actions and their outcomes to avoid unnecessary repetition.",
-        "Avoid unnecessary actions: If you already have enough information to complete the user task, return an empty tool calls array.",
-    ]
+def get_default_instructions():
+    """The default parallel function calling agent instructions."""
+    return """
+Think step by step: Use the thinking field to elaborate what you observe and what do you need to accomplish next.
+Reflect on prior steps: Review your previous actions and their outcomes to avoid unnecessary repetition.
+Avoid unnecessary actions: If you already have enough information to complete the user task, return an empty tool calls array.
+""".strip()
 
 
 @synalinks_export(
@@ -46,7 +40,7 @@ class FunctionCallingAgent(Module):
     The agent has 2 different modes:
 
     - Autonomous: It will execute tools as soon as called.
-    - Non-autonomous: It will return the tool arguments as an ChatMessage.
+    - Non-autonomous: It will return the tool arguments as a ChatMessage.
 
     In *autonomous* mode, the agent accept **any kind of data model input** and perform a final inference to
     eventually format its final answer if a `data_model` or `schema` is provided.
@@ -257,16 +251,17 @@ class FunctionCallingAgent(Module):
             # Validate the tool calls arguments (with an UI or CLI)
             # Then re-inject the validated assistant response in the input_messages
             # The corresponding tools will be called by the agent
-            # Here we assume everything is okay for the purpose of the demo ^^
+            # Here we assume everything is okay for the purpose of the demo.
 
             input_messages.messages.append(assistant_message)
 
     if __name__ == "__main__":
         asyncio.run(main())
     ```
-    
-    The FunctionCallingAgent is compatible with MCP tools, here is an example on how to use it:
-    
+
+    The FunctionCallingAgent is compatible with MCP tools,
+    here is an example on how to use it:
+
     ```python
     import synalinks
     import asyncio
@@ -288,7 +283,7 @@ class FunctionCallingAgent(Module):
 
 
     async def main():
-        
+
         language_model = synalinks.LanguageModel(
             model="ollama/mistral",
         )
@@ -303,7 +298,7 @@ class FunctionCallingAgent(Module):
         )
 
         tools = await mcp_client.get_tools()
-        
+
         inputs = synalinks.Input(data_model=Query)
         outputs = await synalinks.FunctionCallingAgent(
             data_model=FinalAnswer,
@@ -324,7 +319,7 @@ class FunctionCallingAgent(Module):
         response = await agent(input_query)
 
         print(response.prettify_json())
-        
+
 
     if __name__ == "__main__":
         asyncio.run(main())
@@ -338,7 +333,6 @@ class FunctionCallingAgent(Module):
         data_model=None,
         language_model=None,
         prompt_template=None,
-        static_system_prompt=None,
         examples=None,
         instructions=None,
         use_inputs_schema=False,
@@ -359,9 +353,6 @@ class FunctionCallingAgent(Module):
         self.schema = schema
 
         self.prompt_template = prompt_template
-        if not static_system_prompt:
-            static_system_prompt = get_default_static_prompt()
-        self.static_system_prompt = static_system_prompt
 
         if not instructions:
             instructions = get_default_instructions()
@@ -383,40 +374,21 @@ class FunctionCallingAgent(Module):
         self.return_inputs_with_trajectory = return_inputs_with_trajectory
         self.max_iterations = max_iterations
 
-        if self.autonomous:
-            self.tool_calls_generators = []
-            for i in range(self.max_iterations):
-                self.tool_calls_generators.append(
-                    ChainOfThought(
-                        schema=tool_calls_schema,
-                        prompt_template=self.prompt_template,
-                        static_system_prompt=self.static_system_prompt,
-                        examples=self.examples,
-                        instructions=self.instructions,
-                        use_inputs_schema=self.use_inputs_schema,
-                        use_outputs_schema=self.use_outputs_schema,
-                        language_model=self.language_model,
-                        name=self.name + f"_tool_calls_generator_{i}",
-                    )
-                )
-        else:
-            self.tool_calls_generator = ChainOfThought(
-                schema=tool_calls_schema,
-                prompt_template=self.prompt_template,
-                static_system_prompt=self.static_system_prompt,
-                examples=self.examples,
-                instructions=self.instructions,
-                use_inputs_schema=self.use_inputs_schema,
-                use_outputs_schema=self.use_outputs_schema,
-                language_model=self.language_model,
-                name=self.name + "_tool_calls_generator",
-            )
+        self.tool_calls_generator = ChainOfThought(
+            schema=tool_calls_schema,
+            prompt_template=self.prompt_template,
+            examples=self.examples,
+            instructions=self.instructions,
+            use_inputs_schema=self.use_inputs_schema,
+            use_outputs_schema=self.use_outputs_schema,
+            language_model=self.language_model,
+            name=self.name + "_tool_calls_generator",
+        )
 
         if self.schema and self.autonomous:
-            self.final_generator = Generator(
+            self.final_generator = ChainOfThought(
                 schema=self.schema,
                 language_model=self.language_model,
-                static_system_prompt=self.static_system_prompt,
                 instructions=self.instructions,
                 return_inputs=self.return_inputs_with_trajectory,
                 name=self.name + "_final_generator",
@@ -438,7 +410,7 @@ class FunctionCallingAgent(Module):
             if not is_chat_messages(inputs):
                 raise ValueError(
                     "In interactive mode, the FunctionCallingAgent "
-                    "needs an ChatMessages as inputs"
+                    "needs an ChatMessages-like data model as inputs"
                 )
             trajectory = inputs
 
@@ -446,10 +418,15 @@ class FunctionCallingAgent(Module):
 
         if self.autonomous:
             for i in range(self.max_iterations):
-                tool_calls = await self.tool_calls_generators[i](trajectory)
+                tool_calls = await self.tool_calls_generator(trajectory)
 
                 if not tool_calls:
-                    continue
+                    assistant_message = ChatMessage(
+                        role=ChatRole.ASSISTANT,
+                        content="Something went wrong while trying to decide the next action.",
+                    )
+                    agent_messages.append(assistant_message.get_json())
+                    break
 
                 assistant_message = ChatMessage(
                     role=ChatRole.ASSISTANT,
@@ -479,16 +456,25 @@ class FunctionCallingAgent(Module):
 
                 agent_messages.append(assistant_message.get_json())
 
-                tool_results = await asyncio.gather(*tasks)
+                tool_results = await asyncio.gather(*tasks, return_exceptions=True)
                 for j, tool_result in enumerate(tool_results):
                     tool_call_id = tool_calls_ids[j]
-                    agent_messages.append(
-                        ChatMessage(
-                            role=ChatRole.TOOL,
-                            tool_call_id=tool_call_id,
-                            content=tool_result,
-                        ).get_json()
-                    )
+                    if isinstance(tool_result, Exception):
+                        agent_messages.append(
+                            ChatMessage(
+                                role=ChatRole.TOOL,
+                                tool_call_id=tool_call_id,
+                                content="error: %s" % str(tool_result),
+                            ).get_json()
+                        )
+                    else:
+                        agent_messages.append(
+                            ChatMessage(
+                                role=ChatRole.TOOL,
+                                tool_call_id=tool_call_id,
+                                content=tool_result,
+                            ).get_json()
+                        )
 
                 trajectory.update({"messages": agent_messages})
             if self.schema:
@@ -509,16 +495,25 @@ class FunctionCallingAgent(Module):
                         tool_calls_ids.append(tool_call_id)
                         tasks.append(self.tools[tool_name](**tools_arguments))
 
-                    tool_results = await asyncio.gather(*tasks)
+                    tool_results = await asyncio.gather(*tasks, return_exceptions=True)
                     for j, tool_result in enumerate(tool_results):
                         tool_call_id = tool_calls_ids[j]
-                        agent_messages.append(
-                            ChatMessage(
-                                role=ChatRole.TOOL,
-                                tool_call_id=tool_call_id,
-                                content=tool_result,
-                            ).get_json()
-                        )
+                        if isinstance(tool_result, Exception):
+                            agent_messages.append(
+                                ChatMessage(
+                                    role=ChatRole.TOOL,
+                                    tool_call_id=tool_call_id,
+                                    content="error: %s" % str(tool_result),
+                                ).get_json()
+                            )
+                        else:
+                            agent_messages.append(
+                                ChatMessage(
+                                    role=ChatRole.TOOL,
+                                    tool_call_id=tool_call_id,
+                                    content=tool_result,
+                                ).get_json()
+                            )
 
             trajectory.update({"messages": agent_messages})
 
@@ -561,8 +556,7 @@ class FunctionCallingAgent(Module):
 
     async def compute_output_spec(self, inputs, training=False):
         if self.autonomous:
-            for i in range(self.max_iterations):
-                _ = await self.tool_calls_generators[i](inputs)
+            _ = await self.tool_calls_generator(inputs)
             if self.schema:
                 return await self.final_generator(inputs)
             else:
@@ -574,7 +568,7 @@ class FunctionCallingAgent(Module):
             if not is_chat_messages(inputs):
                 raise ValueError(
                     "In interactive mode, the FunctionCallingAgent "
-                    "needs an ChatMessages as inputs"
+                    "needs an ChatMessages-like data model as inputs"
                 )
 
             _ = await self.tool_calls_generator(inputs)
@@ -594,7 +588,6 @@ class FunctionCallingAgent(Module):
         config = {
             "schema": self.schema,
             "prompt_template": self.prompt_template,
-            "static_system_prompt": self.static_system_prompt,
             "examples": self.examples,
             "instructions": self.instructions,
             "use_inputs_schema": self.use_inputs_schema,
