@@ -82,6 +82,8 @@ class Branch(Module):
         examples (list): The default examples to use in the prompt
             (see `Decision`).
         instructions (list): The default instructions to use (see `Decision`).
+        seed_instructions (list): Optional. A list of instructions to use as seed for the
+            optimization. If not provided, use the default instructions as seed.
         temperature (float): Optional. The temperature for the LM call.
         use_inputs_schema (bool): Optional. Whether or not use the inputs
             schema in the decision prompt (Default to False) (see `Decision`).
@@ -104,6 +106,7 @@ class Branch(Module):
         prompt_template=None,
         examples=None,
         instructions=None,
+        seed_instructions=None,
         temperature=0.0,
         use_inputs_schema=False,
         use_outputs_schema=False,
@@ -133,6 +136,7 @@ class Branch(Module):
         self.prompt_template = prompt_template
         self.examples = examples
         self.instructions = instructions
+        self.seed_instructions = seed_instructions
         self.temperature = temperature
         self.use_inputs_schema = use_inputs_schema
         self.use_outputs_schema = use_outputs_schema
@@ -143,6 +147,7 @@ class Branch(Module):
             prompt_template=self.prompt_template,
             examples=self.examples,
             instructions=self.instructions,
+            seed_instructions=self.seed_instructions,
             temperature=self.temperature,
             use_inputs_schema=self.use_inputs_schema,
             use_outputs_schema=self.use_outputs_schema,
@@ -252,6 +257,7 @@ class Branch(Module):
             "prompt_template": self.prompt_template,
             "examples": self.examples,
             "instructions": self.instructions,
+            "seed_instructions": self.seed_instructions,
             "temperature": self.temperature,
             "use_inputs_schema": self.use_inputs_schema,
             "use_outputs_schema": self.use_outputs_schema,
