@@ -114,6 +114,8 @@ class SelfCritique(Module):
             are a list of tuples containing input/output JSON pairs.
         instructions (str): The default instructions being a string containing
             instructions for the language model.
+        seed_instructions (list): Optional. A list of instructions to use as seed for the
+            optimization. If not provided, use the default instructions as seed.
         temperature (float): Optional. The temperature for the LM call.
         use_inputs_schema (bool): Optional. Whether or not use the inputs schema in
             the prompt (Default to False) (see `Generator`).
@@ -133,6 +135,7 @@ class SelfCritique(Module):
         prompt_template=None,
         examples=None,
         instructions=None,
+        seed_instructions=None,
         temperature=0.0,
         use_inputs_schema=False,
         use_outputs_schema=False,
@@ -151,6 +154,7 @@ class SelfCritique(Module):
         self.prompt_template = prompt_template
         self.examples = examples
         self.instructions = instructions
+        self.seed_instructions = seed_instructions
         self.temperature = temperature
         self.use_inputs_schema = use_inputs_schema
         self.use_outputs_schema = use_outputs_schema
@@ -168,6 +172,7 @@ class SelfCritique(Module):
             prompt_template=self.prompt_template,
             examples=self.examples,
             instructions=self.instructions,
+            seed_instructions=self.seed_instructions,
             temperature=self.temperature,
             use_inputs_schema=self.use_inputs_schema,
             use_outputs_schema=self.use_outputs_schema,
@@ -183,6 +188,7 @@ class SelfCritique(Module):
             "prompt_template": self.prompt_template,
             "examples": self.examples,
             "instructions": self.instructions,
+            "seed_instructions": self.seed_instructions,
             "temperature": self.temperature,
             "use_inputs_schema": self.use_inputs_schema,
             "use_outputs_schema": self.use_outputs_schema,
