@@ -9,7 +9,7 @@ import unittest
 from absl.testing import parameterized
 
 from synalinks.src.backend.common.global_state import clear_session
-from synalinks.src.backend.config import disable_telemetry
+from synalinks.src.backend.config import enable_logging
 
 
 class TestCase(
@@ -23,7 +23,7 @@ class TestCase(
     def setUp(self):
         # clear global state so that test cases are independent
         clear_session(free_memory=False)
-        disable_telemetry()
+        enable_logging()
 
     def get_temp_dir(self):
         temp_dir = tempfile.mkdtemp()
