@@ -87,9 +87,9 @@ class Trainer:
                 instance. See `synalinks.metrics`. A function is any callable with the
                 signature `result = fn(y_true, y_pred)`.
             run_eagerly (bool): If `True`, this program's forward pass
-                 will never be compiled. It is recommended to leave this
-                 as `False` when training (for best performance),
-                 and to set it to `True` when debugging.
+                will never be compiled. It is recommended to leave this
+                as `False` when training (for best performance),
+                and to set it to `True` when debugging.
             steps_per_execution (int): The number of batches to run
                 during each a single compiled function call. Running multiple
                 batches inside a single compiled function call can
@@ -329,7 +329,7 @@ class Trainer:
         self,
         x=None,
         y=None,
-        batch_size=None,
+        batch_size=1,
         epochs=1,
         verbose="auto",
         callbacks=None,
@@ -339,7 +339,7 @@ class Trainer:
         initial_epoch=0,
         steps_per_epoch=None,
         validation_steps=None,
-        validation_batch_size=None,
+        validation_batch_size=32,
         validation_freq=1,
     ):
         """Trains the program for a fixed number of epochs (dataset iterations).
@@ -527,6 +527,7 @@ class Trainer:
                     ):
                         await self.optimizer.on_batch_begin(
                             step,
+                            epoch,
                             self.trainable_variables,
                         )
 
@@ -555,6 +556,7 @@ class Trainer:
                     ):
                         await self.optimizer.on_batch_end(
                             step,
+                            epoch,
                             self.trainable_variables,
                         )
 
@@ -596,6 +598,7 @@ class Trainer:
                     epoch,
                     self.trainable_variables,
                 )
+
             callbacks.on_epoch_end(epoch, epoch_logs)
             training_logs = epoch_logs
             if self.stop_training:
@@ -617,7 +620,7 @@ class Trainer:
         self,
         x=None,
         y=None,
-        batch_size=None,
+        batch_size=32,
         verbose="auto",
         steps=None,
         callbacks=None,
@@ -641,7 +644,7 @@ class Trainer:
                 `x`.
             batch_size (int): Integer or `None`.
                 Number of samples per batch of computation.
-                If unspecified, `batch_size` will default to 1.
+                If unspecified, `batch_size` will default to 32.
                 Do not specify the `batch_size` if your input data `x` is a
                 Python generator function since they generate batches.
             verbose (int | str): `"auto"`, 0, 1, or 2. Verbosity mode.
@@ -725,7 +728,7 @@ class Trainer:
             callbacks.on_test_batch_end(step, logs)
             if self.stop_evaluating:
                 break
-        logs = self._get_metrics_result_or_logs(logs)
+        logs = self.get_metrics_result()
         callbacks.on_test_end(logs)
 
         if return_dict:
@@ -896,7 +899,6 @@ class Trainer:
             y_pred=y_pred,
             training=False,
         )
-
         await self._reward_tracker.update_state(reward)
 
         metrics = await self.compute_metrics(x, y, y_pred)
